/**
 * Copyright 2009-2012 Serli
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.geolocation.gwt.client;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

import com.google.gwt.core.client.EntryPoint;
import com.google.gwt.core.client.GWT;
import com.google.gwt.event.dom.client.MouseUpEvent;
import com.google.gwt.event.dom.client.MouseUpHandler;
import com.google.gwt.maps.client.MapType;
import com.google.gwt.maps.client.MapWidget;
import com.google.gwt.maps.client.control.LargeMapControl;
import com.google.gwt.maps.client.geom.LatLng;
import com.google.gwt.maps.client.overlay.Marker;
import com.google.gwt.maps.client.overlay.MarkerOptions;
import com.google.gwt.user.client.Timer;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.ListBox;
import com.google.gwt.user.client.ui.RootPanel;

/**
 * Entry point classes define <code>onModuleLoad()</code>.
 * @author mancelin
 */
public class OW2ProductsUsers implements EntryPoint {
    /**
     * Wait constant.
     */
    private static final int WAIT = 500;
    /**
     * Wait constant.
     */
    private static final int WAIT_REPEATED = 10000;
    /**
     * The latitude of map's center.
     */
    private static final double CENTER_LATITUDE = 33.724340;
    /**
     * Width of the map.
     */
    private static final String WIDTH = "856px";
    /**
     * Height of the map.
     */
    private static final String HEIGHT = "600px";
    /**
     * Common error message.
     */
    private static final String ERROR_MESSAGE = "Ooops, we're experiencing "
            + "technical issues. "
            + "We apologize for the inconvenience.";
    /**
     * The google map widget.
     */
    private MapWidget map;
    /**
     * Filter on products.
     */
    private String oldFilter = "All products";
    /**
     * Filter on products.
     */
    private String filter = "All products";
    /**
     * A listbox widget for products.
     */
    private ListBox tree = new ListBox();
    /**
     * The markers on the map.
     */
    private Vector<Marker> markers = new Vector();
    /**
     * The html widget for number of users.
     */
    private HTML text = new HTML("0");
    /**
     * The html widget for some message.
     */
    private HTML usersText = new HTML("Total users : ");
    /**
     * 
     */
    private HTML title = new HTML("<h1 align =\"center\">OW2 Products users in the World</h1>");
    /**
     * 
     */
    private HTML error = new HTML("");
    /**
     * The center of the map.
     */
    private LatLng centre = LatLng.newInstance(CENTER_LATITUDE, 0);
    /**
     * The remote service to get cities in the DB.
     */
    private final CityGetterServiceAsync cityService = GWT.create(CityGetterService.class);
    /**
     * Async callback to get number of users from GWT servlet.
     */
    private AsyncCallback getUsers = new AsyncCallback<Long>() {

        public void onFailure(final Throwable caught) {
            showError(ERROR_MESSAGE, true);

        }

        public void onSuccess(final Long result) {
            text.setText(result.toString());
        }
    };
    /**
     * Async callback to get managed products from GWT servlet.
     */
    private AsyncCallback getProducts = new AsyncCallback<Vector<String>>() {

        public void onFailure(final Throwable caught) {
            showError(ERROR_MESSAGE, true);
        }

        public void onSuccess(final Vector<String> result) {
            Iterator it = result.iterator();
            while (it.hasNext()) {
                String tmp = (String) it.next();
                addProductItem(tmp);
            }
        }
    };
    /**
     * Async callback to get cities from GWT servlet.
     */
    private AsyncCallback getCities = new AsyncCallback<Vector<HashMap<String, String>>>() {

        public void onFailure(final Throwable caught) {
            showError(ERROR_MESSAGE, true);
        }

        public void onSuccess(final Vector<HashMap<String, String>> result) {
            Iterator it = result.iterator();
            while (it.hasNext()) {
                HashMap<String, String> city = (HashMap<String, String>) it.next();
                addMarker(LatLng.newInstance(Double.valueOf(city.get("longitude")),
                        Double.valueOf(city.get("latitude"))),
                        city.get("address"), city.get("product"));
            }
            cityService.getProducts(getProducts);
            Iterator it2 = markers.iterator();
            while (it2.hasNext()) {
                Marker m = (Marker) it2.next();
                map.addOverlay(m);
            }
            cityService.getUsers(filter, getUsers);
            cityService.getProducts(getProducts);
        }
    };

    /**
     * This method call the city service and add a marker on each city returned.
     */
    public void onModuleLoad() {
        markers = new Vector();
        String askProduct = Window.Location.getParameter("productName");
        if (askProduct != null) {
            filter = askProduct + "**";
        }
        tree.addMouseUpHandler(new MouseUpHandler() {
            public void onMouseUp(final MouseUpEvent event) {
                updateMap(filter);
            }
        });
        tree.addItem("All products");
        map = new MapWidget(centre, 2);
        map.setDoubleClickZoom(true);
        map.setGoogleBarEnabled(true);
        map.setScrollWheelZoomEnabled(true);
        map.setSize(WIDTH, HEIGHT);
        map.addControl(new LargeMapControl());
        map.setCurrentMapType(MapType.getHybridMap());
        RootPanel.get("title").add(title);
        RootPanel.get("error").add(error);
        error.setVisible(false);
        RootPanel.get("map").add(map);
        RootPanel.get("filter").add(tree);
        RootPanel.get("users").add(text);
        RootPanel.get("usersText").add(usersText);
        cityService.getCities(getCities);
        Iterator it = markers.iterator();
        while (it.hasNext()) {
            map.addOverlay((Marker) it.next());
        }
        Timer t = new Timer() {
            public void run() {
                updateMap(filter, false);
            }
        };
        t.schedule(WAIT);
    }

    /**
     * Update the map with the tree filter.
     *
     * @param filterString the filter for the update.
     */
    private void updateMap(final String filterString) {
        updateMap(filterString, true);
    }
    
    /**
     * Display message error.
     * @param errorText the text displayed.
     * @param timed should the error message disappear.
     */
    private void showError(final String errorText, final boolean timed) {
       error.setHTML(errorText);
       error.setVisible(true);
       Timer t = new Timer() {
            public void run() {
                OW2ProductsUsers.this.error.setVisible(false);
            }
        };
        if (timed) {
            t.schedule(WAIT_REPEATED);
        }
    }

    /**
     * Update de visual content of the map.
     *
     * @param filterString the filter for the update.
     * @param getUIFilter
     */
    private void updateMap(String filterString, final boolean getUIFilter) {       
        if (getUIFilter) {
            filterString = tree.getItemText(tree.getSelectedIndex());
        }
        if (!oldFilter.equals(filterString)) {
            Iterator it2 = markers.iterator();
            while (it2.hasNext()) {
                Marker tmp = ((Marker) it2.next());
                if (tmp.getTitle().contains(filterString.replace("**", ""))) {
                    tmp.setVisible(true);
                } else if (filterString.equals("All products")) {
                    tmp.setVisible(true);
                } else {
                    tmp.setVisible(false);
                }
            }
        }
        cityService.getProducts(getProducts);
        cityService.getUsers(filterString, getUsers);
        oldFilter = filterString;
    }

    /**
     * This method add a new marker on the google map.
     * @param point the coordinates of the marker.
     * @param ville the address of the city.
     * @param product the product added.
     */
    public void addMarker(final LatLng point, final String ville, final String product) {
        MarkerOptions opt = MarkerOptions.newInstance();
        opt.setTitle(product);
//        opt.setDraggable(true);
//        Icon icn = Icon.getDefaultIcon();
//        icn.setImageURL("http://wiki.easybeans.org/xwiki/bin/download/XWiki/MySkin/miniitem.png");
//        icn.setIconSize(Size.newInstance(16,15));
//        icn.setShadowSize(Size.newInstance(0,0));
//        icn.setIconAnchor(Point.newInstance(2, 19));
//        opt.setIcon(icn);
        Marker m = new Marker(point, opt);
        if (!markers.contains(m)) {
            markers.add(m);
        }
    }

    /**
     * Add a new product on the map.
     * @param name name of the product.
     */
    public void addProductItem(final String name) {
        boolean contains = false;
        for (int i = 0; i < tree.getItemCount(); i++) {
            if (tree.getItemText(i).equals(name)) {
                contains = true;
            }
        }
        if (!contains) {
            tree.addItem(name);
        }
    }
}
