/**
 * Copyright 2009-2012 Serli
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.geolocation.gwt.models;

import java.io.Serializable;
import java.util.HashMap;

/**
 * This class represent a city (where live Easybeans users).
 * @author mancelin
 */
public class City implements Serializable {

    /**
     * The address of the city.
     */
    private String address;
    /**
     * The longitude of the city.
     */
    private double longitude;
    /**
     * The latitude of the city.
     */
    private double latitude;
    /**
     * The number of users in this city.
     */
    private int users;
    /**
     *
     */
    private HashMap<String, Integer> product;

    /**
     * @return products in the city.
     */
    public HashMap<String, Integer> getProduct() {
        return product;
    }

    /**
     * Add a product to the city.
     * @param name the name of the product.
     */
    public void addProduct(final String name) {
        if (this.product.containsKey(name)) {
            this.product.put(name, ((Integer) this.product.get(name)) + 1);
        } else {
            this.product.put(name, 1);
        }
    }
    /**
     * Set the address of the city.
     * @param address the address of the city.
     */
    public void setAddress(final String address) {
        this.address = address;
    }

    /**
     * The constructor of city.
     * @param address The address of the city
     * @param longitude The longitude of the city
     * @param latitude The latitude of the city
     * @param users The number of users in this city
     * @param product The product.
     */
    public City(
            final String address,
            final double longitude,
            final double latitude,
            final int users,
            final String product) {
        this.address = address;
        this.longitude = longitude;
        this.latitude = latitude;
        this.users = users;
        this.product = new HashMap<String, Integer>();
        this.product.put(product, 1);
    }

    /**
     * @return the address of the city
     */
    public String getAddress() {
        return this.address;
    }

    /**
     * @return the longitude of the city
     */
    public double getLongitude() {
        return this.longitude;
    }
    /**
     * Set the longitude of the city.
     * @param longitude the longitude of the city.
     */
    public void setLongitude(final double longitude) {
        this.longitude = longitude;
    }
    /**
     * Set the latitude of the city.
     * @param latitude the laltitude of the city;
     */
    public void setLatitude(final double latitude) {
        this.latitude = latitude;
    }

    /**
     * @return the latitude of the city
     */
    public double getLatitude() {
        return this.latitude;
    }

    /**
     * @return the number of users the city
     */
    public int getUsers() {
        return this.users;
    }
    /**
     * Set the number of users.
     * @param numb number of user.
     */
    public void setUsers(final int numb) {
        this.users = numb;
    }
}
