/**
 * Copyright 2009-2012 Serli
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.geolocation.gwt.server;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;
import java.util.Map.Entry;

import javax.ejb.EJB;

import org.ow2.util.geolocation.business.GeoLocatorInterface;
import org.ow2.util.geolocation.business.entity.GeolocalisedProduct;
import org.ow2.util.geolocation.gwt.client.CityGetterService;
import org.ow2.util.geolocation.gwt.models.City;

import com.google.gwt.user.server.rpc.RemoteServiceServlet;

/**
 * This servlet provide a remote service to get the cities of the DB.
 * @author mancelin
 */
@SuppressWarnings("serial")
public class CityGetter extends RemoteServiceServlet implements
        CityGetterService {
    /**
     * A null string.
     */
    private static final String NULL = "";
    /**
     * The geolocalisation EJB.
     */
    @EJB
    private GeoLocatorInterface clientSess;
    /**
     * The cities.
     */
    private Vector<City> cities;
    /**
     *
     */
    private Vector<String> registeredProducts;

    /**
     *
     */
    private HashMap<String, Long> usersPerProduct = new HashMap<String, Long>();
    /**
     *
     */
    private long users = 0;
    /**
     *
     */
    private long oldUsers = 0;

    /**
     * Get the cities in the DB and search coordinates if not presents.
     */
    public void buildCities() {
        this.cities = new Vector<City>();
        this.registeredProducts = new Vector<String>();
        this.usersPerProduct = new HashMap<String, Long>();
        try {
            List productsList = this.clientSess.getLocalisedProducts(NULL, NULL);

            for (GeolocalisedProduct p : (List<GeolocalisedProduct>) productsList) {
                String tmpName = p.getProductName() + " " + p.getProductVersion();
                if (this.usersPerProduct.containsKey(tmpName)) {
                    this.usersPerProduct.put(tmpName, Long.valueOf(this.usersPerProduct.get(tmpName) + 1));
                } else {
                    this.usersPerProduct.put(tmpName, Long.valueOf(1));
                }
                City tmpCity = new City(p.getCity(),
                        Double.valueOf(p.getLatitude()),
                        Double.valueOf(p.getLongitude()),
                        1,
                        tmpName);
                HashMap<String, Integer> productForCity = tmpCity.getProduct();
                Iterator prodIt = productForCity.entrySet().iterator();
                while (prodIt.hasNext()) {
                    Entry e = (Entry) prodIt.next();
                    if (!this.registeredProducts.contains(e.getKey())) {
                        this.registeredProducts.add((String) e.getKey());
                    }
                }
                if (!ownCity(tmpCity, tmpName)) {
                    this.cities.add(tmpCity);
                }
            }

        } catch (Exception e) {
            System.out.println("Error while retriving products " + e);
        }
    }
    /**
     * Set managed cities.
     * @param cities managed cities.
     */
    public void setCities(final Vector<City> cities) {
        this.cities = cities;
    }
    /**
     * @param cit the city.
     * @param name name of the product.
     * @return if a managed city own a product.
     */
    private boolean ownCity(final City cit, final String name) {
        for (City city : this.cities) {
            if ((city.getLatitude() == cit.getLatitude()) && (city.getLongitude() == cit.getLongitude())) {
                city.addProduct(name);
                return true;
            }
        }
        return false;
    }
    /**
     * The remote service method used from the GWT view to get
     * the cities to mark. (I used vector of hashmap 'cause
     * GWT seems not to like a POJO as parameter).
     * @return managed cities.
     */
    public Vector<HashMap<String, String>> getCities() {
        buildCities();
        Vector<HashMap<String, String>> ret = new Vector<HashMap<String, String>>();
        Iterator<City> it = this.cities.iterator();
        this.users = 0;
        while (it.hasNext()) {
            HashMap<String, String> city = new HashMap<String, String>();
            City c = it.next();
            city.put("address", c.getAddress());
            String product = "";
            for (Entry<String, Integer> e : c.getProduct().entrySet()) {
                product += String.valueOf(e.getValue()) + " [" + e.getKey() + "] user(s), \n";
                this.users += e.getValue();
            }
            product += " in " + c.getAddress();
            city.put("product", product);
            city.put("users", String.valueOf(c.getUsers()));
            city.put("latitude", String.valueOf(c.getLatitude()));
            city.put("longitude", String.valueOf(c.getLongitude()));
            if (c.getAddress() != null) {
                if (!product.equals(" in " + c.getAddress()) && !c.getAddress().equals("null")) {
                    ret.add(city);
                }
            }
        }
        this.oldUsers = this.users;
        return ret;
    }
    /**
     * @return the registered products.
     */
    public Vector<String> getProducts() {
        return this.registeredProducts;
    }
    /**
     * @param filter product filter
     * @return number of users for a product.
     */
    public Long getUsers(final String filter) {
        if (filter.equals("All products")) {
            return oldUsers;
        }
        if (filter.contains("**")) {
            long ret = 0;
            for (String product : usersPerProduct.keySet()) {
                if (product.contains(filter.replace("**", ""))) {
                    ret += usersPerProduct.get(product);
                }
            }
            return ret;
        } else {
            return this.usersPerProduct.get(filter);
        }
    }
}
